@@ -17,32 +17,33 @@ class Frame:
     
     # set default values
     prefix = ''
-    ccflags, fcflags, noopt = ['-O3','-O3','']
-    mpibindir = ''               # directory containing mpicc and mpif77
-    mpiincdir = ''               # directory containing mpi.h
-    mpicc, mpif77 = ['', '']     # the mpicc and mpif77 commands
-    make         = 'make'        # the "make" command
+    ccflags, fcflags, f90flags, noopt = ['-O3','-O3','-O3', '']
+    mpibindir = ''                           # directory containing mpicc and mpif77
+    mpiincdir = ''                           # directory containing mpi.h
+    mpicc, mpif77 = ['', '']                 # the mpicc and mpif77 commands
+    f90          = ''                        # the F90 compiler for lapack
+    make         = 'make'                    # the "make" command
     plat         = 'LINUX'       
-    mangling     = ''            # sets the mangling type for Fortran to C calls
-    blaslib      = ''            # the blas library
-    blacslib     = ''            # the blacs library
-    blacsClib    = ''            # the blacs C library
-    blacsF77lib  = ''            # the blacs F77 library
-    lapacklib    = ''            # lapack library
-    scalapacklib = ''            # scalapack library
-    proxy        = 0
-    downblas     = 0             # whether or not to download reference blas
-    downblacs    = 1             # whether or not to download reference blacs
-    downlapack   = 0             # whether or not to download reference lapack
-    ldflags_c    = ''            # loader flags when main program is in C
-    ldflags_f77  = ''            # loader flags when main program is in Fortran
-    testing      = 1             # whether on not to compile and run LAPACK and ScaLAPACK test programs
-    downcmd      = ''            # the command used to download stuff
+    mangling     = ''                        # sets the mangling type for Fortran to C calls
+    blaslib      = ''                        # the blas library
+    blacslib     = ''                        # the blacs library
+    blacsClib    = ''                        # the blacs C library
+    blacsF77lib  = ''                        # the blacs F77 library
+    lapacklib    = ''                        # lapack library
+    scalapacklib = ''                        # scalapack library
+    proxy        = 0 
+    downblas     = 0                         # whether or not to download reference blas
+    downblacs    = 1                         # whether or not to download reference blacs
+    downlapack   = 0                         # whether or not to download reference lapack
+    ldflags_c    = ''                        # loader flags when main program is in C
+    ldflags_f77  = ''                        # loader flags when main program is in Fortran
+    testing      = 1                         # whether on not to compile and run LAPACK and ScaLAPACK test programs
+    downcmd      = ''                        # the command used to download stuff
     blasurl      = 'http://netlib.org/blas/blas.tgz'
     lapackurl    = 'http://netlib.org/lapack/lapack.tgz'
     blacsurl     = 'http://netlib.org/blacs/mpiblacs.tgz'
     scalapackurl = 'http://netlib.org/scalapack/scalapack-1.8.0.tgz'
-    ranlib       = ''            # the ranlib command
+    ranlib       = ''                        # the ranlib command
     clean        = 0
 
     def __init__(self, argv):
@@ -73,6 +74,15 @@ the installation. You can specify the using the --mpibindir
 or the --mpicc and --mpif77 flags."""
                 sys.exit()
                 
+        if((Frame.f90 == '') and (Frame.downlapack == 1)):
+            #  if no F90 compiler is provided, and lapack has to be compiled,
+            #  raise an error
+            print """
+A Fortran 90/95 compiler is required to install the LAPACK library.
+You can specify it using the --f90 flag."""
+            sys.exit()
+                
+
         if(Frame.mpiincdir==''):
             # the path to the mpi.h file is needed
             if(not self.look_for_mpih()):
@@ -115,6 +125,8 @@ needed in the case where testing is disabled by means of the
         # CUSTOM CHECKS
         self.check_mpicc()
         self.check_mpif77()
+        if(Frame.downlapack==1):
+            self.check_f90()
         self.set_mangling()
         self.set_download()
         self.set_ranlib()
@@ -150,6 +162,10 @@ needed in the case where testing is disabled by means of the
           --mpif77=[CMD]       : the mpif77 command.
                                  (default: will take the first available mpif77 in the path)
 
+          --f90=[CMD]          : the Fortran 90/95 compiler.
+                                 A Fortran 90/95 is required only in the case where LAPACK has to be
+                                 installed, i.e., the --downlapack flag was provided.
+
           --mpiincdir=[DIR]    : the path to the directory containing mpi.h
                                  (default: will take the corresponding mpi.h to the mpicc/mpif77 found)
 
@@ -159,6 +175,9 @@ needed in the case where testing is disabled by means of the
           --fcflags=[FLAGS]    : the flags for the F77 compiler
                                  (default -O3)
 
+          --f90flags=[FLAGS]   : the flags for the F90/95 compiler
+                                 (default -O3)
+
           --noopt=[FLAGS]      : compilation flags to be used
                                  on machine dependent subroutines
                                  in LAPACK and ScaLAPACK.
@@ -168,7 +187,11 @@ needed in the case where testing is disabled by means of the
                                  different options when linking C main programs to Fortran subroutines
                                  and vice-versa
 
-          --ldflags_f77=[flags]: loader flags when main program is in Fortran. Some compilers (e.g. PGI) require 
+          --ldflags_f77=[flags]: loader flags when main program is in Fortran 77. Some compilers (e.g. PGI) require 
+                                 different options when linking Fortran main programs to C subroutines
+                                 and vice-versa
+
+          --ldflags_f90=[flags]: loader flags when main program is in Fortran 90/95. Some compilers (e.g. PGI) require 
                                  different options when linking Fortran main programs to C subroutines
                                  and vice-versa
 
@@ -217,8 +240,8 @@ Note: If you want use a proxy for downloading, the http_proxy environment variab
 
         try:
           opts, args = getopt.getopt(sys.argv[1:], "h", ["help","prefix=",
-          "ccflags=", "fcflags=", "noopt=", "makecmd=", "mpibindir=", "mpiincdir=", "blacslib=",
-          "blacsclib=", "blacsf77lib=", "blaslib=", "lapacklib=", "ldflags_c=", "ldflags_f77=","mpicc=","mpif77=",
+          "ccflags=", "fcflags=", "f90flags", "noopt=", "makecmd=", "mpibindir=", "mpiincdir=", "blacslib=",
+          "blacsclib=", "blacsf77lib=", "blaslib=", "lapacklib=", "ldflags_c=", "ldflags_f77=","mpicc=","mpif77=","f90=",
           "downblas", "downblacs", "downlapack", "notesting","clean"])
         except getopt.error, msg:
           print msg
@@ -241,7 +264,10 @@ Note: If you want use a proxy for downloading, the http_proxy environment variab
                 print 'C flags are ', a
             elif(o=='--fcflags'):
                 Frame.fcflags = a
-                print 'Fortran flags are ', a
+                print 'Fortran 77 flags are ', a
+            elif(o=='--f90flags'):
+                Frame.f90flags = a
+                print 'Fortran 90/95 flags are ', a
             elif(o=='--noopt'):
                 Frame.noopt = a
                 print 'NOOPT flags are ', a
@@ -259,6 +285,9 @@ Note: If you want use a proxy for downloading, the http_proxy environment variab
             elif(o=='--mpif77'):
                 Frame.mpif77 = a
                 print 'mpif77 is ', a
+            elif(o=='--f90'):
+                Frame.f90 = a
+                print 'The Fortran 90/95 compiler is ', a
             elif(o == '--blacslib'):
                 Frame.blacslib = fixpaths(a)
                 Frame.downblacs = 0
@@ -378,7 +407,7 @@ in the PATH environment variable.
             }\n""")
 
         # compile
-        ccomm = Frame.mpicc+' -o tmpc '+os.path.join(os.getcwd(),'tmpc.c')
+        ccomm = Frame.mpicc+' '+Frame.ccflags+' -o tmpc '+os.path.join(os.getcwd(),'tmpc.c')
         (output, error, retz) = runShellCommand(ccomm)
       
         if(retz != 0):
@@ -423,7 +452,7 @@ in the PATH environment variable.
       end\n""")
 
         # compile
-        fcomm = Frame.mpif77+' -o tmpf '+'tmpf.f'
+        fcomm = Frame.mpif77+' '+Frame.fcflags+' -o tmpf '+'tmpf.f'
         (output, error, retz) = runShellCommand(fcomm)
 
         if(retz != 0):
@@ -446,6 +475,46 @@ in the PATH environment variable.
         return 0;
         
 
+    def check_f90(self):
+        """ check if the Fortran 90/95 compiler works """
+        # simply generates a F90 program and checks if the compilation and execution
+        # are succesful
+        print 'Checking if the Fortran 90/95 compiler works...',
+        sys.stdout.flush()
+        # generate        
+        writefile('tmpf.f90',"""
+program testf90
+   integer, pointer :: a(:)
+   allocate(a(10))
+   a(:)=1
+   deallocate(A)
+   stop
+end program testf90\n""")
+
+        # compile
+        fcomm = Frame.f90+' '+Frame.f90flags+' -o tmpf '+'tmpf.f90'
+        (output, error, retz) = runShellCommand(fcomm)
+
+        if(retz != 0):
+            print '\n\nCOMMON: the Fortran 90/95 compiler is not working! aborting...'
+            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
+            sys.exit()
+        
+        # run
+        comm = './tmpf'
+        (output, error, retz) = runShellCommand(comm)
+        if(retz != 0):
+            print '\n\nCOMMON: the Fortran 90/95 compiler is not working! aborting...'
+            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
+            sys.exit()
+
+        # cleanup        
+        killfiles(['tmpf.f90','tmpf','tmpf.o'])
+        print 'yes'
+
+        return 0;
+        
+
     def set_mangling(self):
         """ Sets the INTFACE variable in Bmake.inc """
         # This one generates a program equivalent to that in BLACS/INSTALL
@@ -665,7 +734,7 @@ in the PATH environment variable.
         # in some cases. This function tries to detect which compilers are used
         # and sets the flags accordingly
 
-        print 'Detecting Fortran compiler...',
+        print 'Detecting Fortran 77 compiler...',
         if (self.fc_is_intel()):
             # The Intel FORTRAN compiler requires -nofor_main flag
             # for the linking and the -mp flag to maintain the 
@@ -682,7 +751,6 @@ in the PATH environment variable.
         else:
             print 'unknown'
 
-
         print 'Detecting C compiler...',
         if (self.cc_is_intel()):
             print 'Intel'
